@@ -44,6 +44,7 @@
 #include "namespaces.h"
 #include "templates.h"
 #include "imports.h"
+#include "transform.h"
 
 #define DEBUG_ATTRIBUTES
 
@@ -368,95 +369,6 @@ error:
         xmlFree(prefix);
 }
 
-/**
- * xsltAttribute:
- * @ctxt:  a XSLT process context
- * @node:  the node in the source tree.
- * @inst:  the xslt attribute node
- *
- * Process the xslt attribute node on the source node
- */
-void
-xsltAttribute(xsltTransformContextPtr ctxt, xmlNodePtr node,
-	           xmlNodePtr inst) {
-    xmlChar *prop = NULL;
-    xmlChar *ncname = NULL;
-    xmlChar *prefix = NULL;
-    xmlChar *value = NULL;
-    xmlNsPtr ns = NULL;
-    xmlAttrPtr attr;
-
-
-    if (ctxt->insert == NULL)
-	return;
-    if (ctxt->insert->children != NULL) {
-	xsltGenericError(xsltGenericErrorContext,
-	     "xslt:attribute : node has already children\n");
-	return;
-    }
-    prop = xsltEvalAttrValueTemplate(ctxt, inst, (const xmlChar *)"name");
-    if (prop == NULL) {
-	xsltGenericError(xsltGenericErrorContext,
-	     "xslt:attribute : name is missing\n");
-	goto error;
-    }
-
-    ncname = xmlSplitQName2(prop, &prefix);
-    if (ncname == NULL) {
-	ncname = prop;
-	prop = NULL;
-	prefix = NULL;
-    }
-
-    if ((prefix != NULL) && (xmlStrEqual(prefix, (const xmlChar *)"xmlns"))) {
-#ifdef DEBUG_ATTRIBUTES
-	xsltGenericDebug(xsltGenericDebugContext,
-	     "xslt:attribute : xmlns prefix forbidden\n");
-#endif
-	goto error;
-    }
-    prop = xsltEvalAttrValueTemplate(ctxt, inst, (const xmlChar *)"namespace");
-    if (prop != NULL) {
-	ns = xsltGetSpecialNamespace(ctxt, inst, prop, prefix, ctxt->insert);
-    } else {
-	if (prefix != NULL) {
-	    ns = xmlSearchNs(inst->doc, inst, prefix);
-	    if (ns == NULL) {
-		xsltGenericError(xsltGenericErrorContext,
-		    "no namespace bound to prefix %s\n", prefix);
-	    } else {
-		ns = xsltGetNamespace(ctxt, inst, ns, ctxt->insert);
-	    }
-	}
-    }
-    
-
-    value = xsltEvalTemplateString(ctxt, node, inst);
-    if (value == NULL) {
-	if (ns) {
-	    attr = xmlSetNsProp(ctxt->insert, ns, ncname, 
-		                (const xmlChar *)"");
-	} else
-	    attr = xmlSetProp(ctxt->insert, ncname, (const xmlChar *)"");
-    } else {
-	if (ns) {
-	    attr = xmlSetNsProp(ctxt->insert, ns, ncname, value);
-	} else
-	    attr = xmlSetProp(ctxt->insert, ncname, value);
-	
-    }
-
-error:
-    if (prop != NULL)
-        xmlFree(prop);
-    if (ncname != NULL)
-        xmlFree(ncname);
-    if (prefix != NULL)
-        xmlFree(prefix);
-    if (value != NULL)
-        xmlFree(value);
-}
-
 /**
  * xsltApplyAttributeSet:
  * @ctxt:  the XSLT stylesheet
@@ -504,7 +416,7 @@ xsltApplyAttributeSet(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    while (style != NULL) {
 		values = xmlHashLookup2(style->attributeSets, ncname, prefix);
 		while (values != NULL) {
-		    xsltAttribute(ctxt, node, values->attr);
+		    xsltAttribute(ctxt, node, values->attr, NULL);
 		    values = values->next;
 		}
 		style = xsltNextImport(style);
