@@ -289,6 +289,21 @@ xsltReverseCompMatch(xsltCompMatchPtr comp) {
     comp->steps[comp->nbStep++].op = XSLT_OP_END;
 }
 
+/**
+ * xsltCleanupCompMatch:
+ * @comp:  the compiled match expression
+ *
+ * remove all computation state from the pattern
+ */
+void
+xsltCleanupCompMatch(xsltCompMatchPtr comp) {
+    int i;
+    
+    for (i = 0;i < comp->nbStep;i++) {
+	comp->steps[i].previous = NULL;
+    }
+}
+
 /************************************************************************
  * 									*
  * 		The interpreter for the precompiled patterns		*
@@ -1016,6 +1031,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
     xmlChar *prefix = NULL;
     xmlChar *ncname = NULL;
     xmlChar *URL = NULL;
+    int level;
 
     SKIP_BLANKS;
     if ((token == NULL) && (CUR == '@')) {
@@ -1152,15 +1168,26 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
     }
 parse_predicate:
     SKIP_BLANKS;
+    level = 0;
     while (CUR == '[') {
 	const xmlChar *q;
 	xmlChar *ret = NULL;
 
+	level++;
 	NEXT;
 	q = CUR_PTR;
 	/* TODO: avoid breaking in strings ... */
-	while ((IS_CHAR(CUR)) && (CUR != ']'))
+	while (IS_CHAR(CUR)) {
+	    /* Skip over nested predicates */
+	    if (CUR == '[')
+		level++;
+	    if (CUR == ']') {
+		level--;
+		if (level == 0)
+		    break;
+	    }
 	    NEXT;
+	}
 	if (!IS_CHAR(CUR)) {
 	    xsltGenericError(xsltGenericErrorContext,
 		    "xsltCompileStepPattern : ']' expected\n");
@@ -1271,7 +1298,7 @@ xsltCompileLocationPathPattern(xsltParserContextPtr ctxt) {
 	    return;
 	}
 	SKIP_BLANKS;
-	if (CUR == '(') {
+	if ((CUR == '(') && !xmlXPathIsNodeType(name)) {
 	    xsltCompileIdKeyPattern(ctxt, name, 1);
 	    if ((CUR == '/') && (NXT(1) == '/')) {
 		PUSH(XSLT_OP_ANCESTOR, NULL, NULL);
@@ -1743,6 +1770,22 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
     return(NULL);
 }
 
+/**
+ * xsltCleanupTemplates:
+ * @style: an XSLT stylesheet
+ *
+ * Cleanup the state of the templates used by the stylesheet and
+ * the ones it imports.
+ */
+void
+xsltCleanupTemplates(xsltStylesheetPtr style) {
+    while (style != NULL) {
+	xmlHashScan((xmlHashTablePtr) style->templatesHash,
+		    (xmlHashScanner) xsltCleanupCompMatch, NULL);
+
+	style = xsltNextImport(style);
+    }
+}
 
 /**
  * xsltFreeTemplateHashes:
